import { InitializeElementNode, ElementDetails, ElementNodeDescriptor } from './nodes/dom';
import { ComponentFactory, ComponentDetails } from './nodes/component';
import { ChildDeclaration, InitializeNode, NodeDescriptor } from './nodes/node';
import { createEventStream, attachEventStream } from './eventing';
import { Stream } from 'most';
export interface UriMap {
    [s: string]: string;
}
export interface StreamDomContext {
    document: HTMLDocument;
    eventNamespaceName: string;
    propertyNamespaceName: string;
    getNamespaceUri(namespaceName: string): string;
    defaultNamespaceUri: string;
    sharedRange: Range;
}
export interface StreamDomScope {
    parentNamespaceUri: string;
    mounted$: Stream<null>;
    destroy$: Stream<null>;
}
export declare const defaultNamespaceUriMap: UriMap;
export declare class StreamDom implements StreamDomContext {
    document: HTMLDocument;
    eventNamespaceName: string;
    propertyNamespaceName: string;
    namespaceUriMap: UriMap;
    defaultNamespaceUri: string;
    sharedRange: Range;
    constructor({document, eventNamespaceName, propertyNamespaceName, namespaceUriMap, defaultNamespaceName}?: {
        document?: Document;
        eventNamespaceName?: string;
        propertyNamespaceName?: string;
        namespaceUriMap?: UriMap;
        defaultNamespaceName?: string;
    });
    getNamespaceUri(namespaceName: string): string;
    mount(streamDomNodeInit: InitializeElementNode, domParentNode: Element, domBeforeNode?: Node): {
        nodeDescriptor: ElementNodeDescriptor;
        dispose(): void;
    };
    text(str: string): InitializeNode;
    element(name: string, details: ElementDetails): InitializeElementNode;
    stream(children$: Stream<ChildDeclaration>): InitializeNode;
    component(ComponentFactory: ComponentFactory, details: ComponentDetails): (scope: StreamDomScope) => NodeDescriptor;
    expression(value: any): InitializeNode | InitializeNode[];
}
export declare function configureStreamDom(config?: {}): StreamDom;
declare const streamDom: StreamDom;
export default streamDom;
export { createEventStream, attachEventStream };
