"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var stream_1 = require('./stream');
var util_1 = require('./util');
var eventing_1 = require('../eventing');
var dom_event_1 = require('@most/dom-event');
function element(context, name, _a) {
    var _b = _a === void 0 ? {} : _a, _c = _b.namespaceName, namespaceName = _c === void 0 ? null : _c, _d = _b.attributes, attributes = _d === void 0 ? [] : _d, _e = _b.children, children = _e === void 0 ? [] : _e;
    return function (scope) {
        var document = context.document;
        var parentNamespaceUri = scope.parentNamespaceUri, mounted$ = scope.mounted$, destroy$ = scope.destroy$;
        var namespaceUri = namespaceName ? context.getNamespaceUri(namespaceName) : parentNamespaceUri;
        var domNode = document.createElementNS(namespaceUri, name);
        processAttributes(context, domNode, attributes, scope);
        var fragment = document.createDocumentFragment();
        var childScope = {
            parentNamespaceUri: namespaceUri,
            mounted$: scope.mounted$,
            destroy$: scope.destroy$
        };
        var childDescriptors = util_1.initializeChildren(children, childScope);
        childDescriptors.forEach(function (cd) { return cd.insert(fragment); });
        domNode.appendChild(fragment);
        return new ElementNodeDescriptor({ domNode: domNode, childDescriptors: childDescriptors });
    };
}
exports.element = element;
function text(context, str) {
    return function () { return new TextNodeDescriptor(context.document.createTextNode(str)); };
}
exports.text = text;
function processAttributes(context, domNode, attributes, scope) {
    attributes.forEach(function (attributeDescriptor) {
        if (Array.isArray(attributeDescriptor)) {
            processAttributes(context, domNode, attributeDescriptor, scope);
        }
        else {
            var namespace = attributeDescriptor.namespace;
            var handler = namespace === context.eventNamespaceName ? handleEventAttribute :
                namespace === context.propertyNamespaceName ? handlePropertyAttribute :
                    handleAttribute;
            handler(context, domNode, attributeDescriptor, scope);
        }
    });
}
function setWithStream(valueOrStream, setter, _a) {
    var destroy$ = _a.destroy$;
    stream_1.isStream(valueOrStream)
        ? valueOrStream.skipRepeats().until(destroy$).observe(setter)
        : setter(valueOrStream);
}
function handleAttribute(context, elementNode, _a, scope) {
    var namespace = _a.namespace, name = _a.name, value = _a.value;
    var namespaceUri = namespace ? context.getNamespaceUri(namespace) : null;
    setWithStream(value, setAttribute, scope);
    function setAttribute(effectiveValue) {
        // Attributes with a no value are treated as boolean
        effectiveValue === null && (effectiveValue = true);
        if (effectiveValue === true || effectiveValue === false) {
            value
                ? elementNode.setAttributeNS(namespaceUri, name, '')
                : elementNode.removeAttributeNS(namespaceUri, name);
        }
        else {
            elementNode.setAttributeNS(namespaceUri, name, effectiveValue);
        }
    }
}
function handlePropertyAttribute(context, elementNode, _a, scope) {
    var name = _a.name, value = _a.value;
    setWithStream(value, setProperty, scope);
    function setProperty(effectiveValue) {
        elementNode[name] = effectiveValue;
    }
}
function handleEventAttribute(context, elementNode, _a, _b) {
    var name = _a.name, proxy$ = _a.value;
    var mounted$ = _b.mounted$, destroy$ = _b.destroy$;
    if (name === 'mount') {
        eventing_1.attachEventStream(proxy$, mounted$.map(function () { return ({ target: elementNode }); }));
    }
    else if (name === 'destroy') {
        eventing_1.attachEventStream(proxy$, destroy$.map(function () { return ({ target: elementNode }); }));
    }
    else {
        eventing_1.attachEventStream(proxy$, dom_event_1.domEvent(name, elementNode).until(destroy$));
    }
}
var DomNodeDescriptor = (function () {
    function DomNodeDescriptor(domNode) {
        this.domNode = domNode;
    }
    Object.defineProperty(DomNodeDescriptor.prototype, "type", {
        get: function () { },
        enumerable: true,
        configurable: true
    });
    DomNodeDescriptor.prototype.insert = function (domParentNode, domBeforeNode) {
        if (domBeforeNode === void 0) { domBeforeNode = null; }
        domParentNode.insertBefore(this.domNode, domBeforeNode);
    };
    DomNodeDescriptor.prototype.remove = function () {
        var domNode = this.domNode;
        domNode.parentNode.removeChild(domNode);
    };
    return DomNodeDescriptor;
}());
exports.DomNodeDescriptor = DomNodeDescriptor;
var ElementNodeDescriptor = (function (_super) {
    __extends(ElementNodeDescriptor, _super);
    function ElementNodeDescriptor(_a) {
        var domNode = _a.domNode, childDescriptors = _a.childDescriptors;
        _super.call(this, domNode);
        this.childDescriptors = childDescriptors;
    }
    Object.defineProperty(ElementNodeDescriptor.prototype, "type", {
        get: function () { return 'element'; },
        enumerable: true,
        configurable: true
    });
    return ElementNodeDescriptor;
}(DomNodeDescriptor));
exports.ElementNodeDescriptor = ElementNodeDescriptor;
var TextNodeDescriptor = (function (_super) {
    __extends(TextNodeDescriptor, _super);
    function TextNodeDescriptor() {
        _super.apply(this, arguments);
    }
    Object.defineProperty(TextNodeDescriptor.prototype, "type", {
        get: function () { return 'text'; },
        enumerable: true,
        configurable: true
    });
    return TextNodeDescriptor;
}(DomNodeDescriptor));
exports.TextNodeDescriptor = TextNodeDescriptor;
