"use strict";
var most_1 = require('most');
var expression_1 = require('./expression');
var util_1 = require('./util');
var symbol_observable_1 = require('symbol-observable');
function stream(context, children$) {
    return function (scope) {
        var document = context.document;
        var mounted$ = scope.mounted$, destroy$ = scope.destroy$;
        var domStartNode = document.createComment('');
        var domEndNode = document.createComment('');
        var childDescriptors$ = children$
            .until(destroy$)
            .map(function (children) {
            var childScope = {
                parentNamespaceUri: scope.parentNamespaceUri,
                mounted$: mounted$,
                destroy$: most_1.merge(children$, destroy$).take(1).multicast()
            };
            return util_1.initializeChildren([expression_1.expression(context, children)], childScope);
        })
            .tap(function (childDescriptors) {
            var document = context.document, sharedRange = context.sharedRange;
            var fragment = document.createDocumentFragment();
            childDescriptors.forEach(function (childDescriptor) { return childDescriptor.insert(fragment); });
            sharedRange.setStartAfter(domStartNode);
            sharedRange.setEndBefore(domEndNode);
            sharedRange.deleteContents();
            sharedRange.insertNode(fragment);
        })
            .multicast();
        childDescriptors$.drain();
        return new StreamNodeDescriptor(context.sharedRange, domStartNode, domEndNode, childDescriptors$);
    };
}
exports.stream = stream;
var StreamNodeDescriptor = (function () {
    function StreamNodeDescriptor(sharedRange, domStartNode, domEndNode, childDescriptors$) {
        this.sharedRange = sharedRange;
        this.domStartNode = domStartNode;
        this.domEndNode = domEndNode;
        this.childDescriptors$ = childDescriptors$;
    }
    Object.defineProperty(StreamNodeDescriptor.prototype, "type", {
        get: function () { return 'stream'; },
        enumerable: true,
        configurable: true
    });
    StreamNodeDescriptor.prototype.insert = function (domParentNode, domBeforeNode) {
        if (domBeforeNode === void 0) { domBeforeNode = null; }
        domParentNode.insertBefore(this.domStartNode, domBeforeNode);
        domParentNode.insertBefore(this.domEndNode, domBeforeNode);
    };
    StreamNodeDescriptor.prototype.remove = function () {
        var sharedRange = this.sharedRange;
        sharedRange.setStartBefore(this.domStartNode);
        sharedRange.setEndAfter(this.domEndNode);
        sharedRange.deleteContents();
    };
    return StreamNodeDescriptor;
}());
function isStream(candidate) {
    return candidate && !!candidate[symbol_observable_1.default];
}
exports.isStream = isStream;
